# Changelog
**Summary of changes of the command line tool**

## v4.0
- **[ FEATURE ]** Allow summaries behind entries to be continued on
  the next line (with increased indentation level), e.g.:
  ```
  2020-01-01
  Both of the following is fine:
      15:00-16:00 This is a very long text, so
          it can be continued on the next line.
      16:00-17:00
          Or, you can just start the entry summary
          on the next line, if you like.
  ```
  The CLI also handles this automatically when it encounters
  line breaks (`\n`), e.g. in the `--summary` flag value.
- **[ FEATURE ]** Add new command `klog pause` that “pauses”
  open-ended time ranges by adding a subsequent pause entry.
- **[ FEATURE ]** Provide rounding option for `klog start` and
  `klog stop`, which rounds times to the nearest multiple of
  5m, 10m, 15m, 30m, or 60m. E.g. for `--round=15m`: `8:03` -> `8:00`.
- **[ FEATURE ]** Add more shortcut filters, e.g. `--this-week`,
  `--last-month`, etc.
- **[ FEATURE ]** Embed the most recent part of the changelog for
  convenience, via `klog --changelog`.
- **[ BREAKING ]** Remove embedded macOS systray widget

## v3.3
- **[ FEATURE ]** Allow times to be `24:00`, e.g. `22:00 - 24:00`.
- **[ FEATURE ]** Add `klog goto` command for opening the file explorer
  at the location of a file or bookmark.
- **[ FEATURE ]** Add `klog bookmark info` command.
- **[ FEATURE ]** When using the manipulation commands (`klog track`, etc.),
  conform to style preferences encountered in the file.
- **[ FEATURE ]** Add `--tomorrow` as shorthand flag for the next day’s date.
- **[ FEATURE ]** Improve warnings (which are shown for potential data problems).
- **[ FIX ]** Fix bug in week-based aggregation of `klog report --aggregate week`.

## v3.2
- **[ BREAKING ]** Don’t allow mixing the indentation style within a
  record. (It might still differ *between* records, though.) For example: if
  the first entry is indented with a tab, then all further entries of that
  particular record have to be indented with a tab as well. In order to check
  that your existing files conform, you can parse all your `.klg` files at once
  via a wildcard lookup, in order to see whether any indentation-related
  errors are reported. On Linux, e.g.: `klog total ~/**/*.klg`.
- **[ FEATURE ]** Allow version check via `klog -v` (in addition
  to `klog --version` or `klog version`)
- **[ FEATURE ]** Embed specification and license in the binary
  (via `klog --spec` and `klog --license`)
- **[ FEATURE ]** Provide binaries for M1 Macs (ARM) for download.
- **[ FIX ]** Fix default sort order of `--sort` flag to be `asc`
- **[ INFO ]** Deprecate the embedded native widget (for MacOS). It will be
  removed in one of the next releases.

## v3.1
- **[ FIX ]** Fix stdin processing on Windows

## v3.0
- **[ FEATURE ]** Support multiple (named) bookmarks to quickly
  reference often-used files, e.g. `klog total @work`
- **[ FEATURE ]** Add additional evaluation options for `klog report`
  to aggregate the data by day, week, month, quarter or year
- **[ FEATURE ]** Add `klog edit` command for opening a file in an editor
  (Based on the `$EDITOR` variable.)
- **[ FEATURE ]** Allow value of `--sort` flag to be uppercase
  or lowercase (`ASC`/`asc` or `DESC`/`desc`)
- **[ FEATURE ]** Support `klog --version` in addition to `klog version`
- **[ FIX ]** Windows: don’t require admin privileges for setting bookmarks 

## v2.6
- **[ INFO ]** Release first version of the file format
  specification (v1.0)
- **[ FIX ]** If a duration only contains a minute part,
  allow the value to be greater than 59, e.g. `120m`.

## v2.5
- **[ BREAKING ]** Rename `klog now` to `klog today`; restructure the
  output, especially when using the `--diff`/`--now` flag
- **[ FEATURE ]** Use distinct exit codes for different error cases
- **[ FEATURE ]** Introduce `--quiet` flag to retrieve raw output
- **[ FEATURE ]** Extend help texts, improve error messages 
- **[ FIX ]** Fix formatting issues of error output

## v2.4
- **[ FEATURE ]** Automatically create a new record when doing
  `klog start` or `klog track` if there is no record yet
- **[ FEATURE ]** Allow wildcard searching in tags by appending `...`,
  e.g. `--tag=foo...` would match the tag `#foobar`
- **[ FIX ]** `klog stop` now also looks for open ranges of the
  previous day and closes them with a shifted end time
- **[ FIX ]** `klog stop --summary=""` doesn’t fail if the existing
  entry summary was empty

## v2.3
- **[ FEATURE ]** Add `--summary`/`-s` flag for `start` and
  `stop` subcommands
- **[ FEATURE ]** If `KLOG_DEBUG` environment variable is set,
  print more verbose error output
- **[ FIX ]** Ensure that reading from stdin works on Windows
- **[ FIX ]** Display a more helpful error message on Windows
  to explain the quirks with `bookmark set`

## v2.2
- **[ FEATURE ]** Provide `--no-style` option to disable output
  formatting (i.e. no colours, underlined, bold, etc.)
- **[ FIX ]** Make sure that output formatting works on Windows
  across all Terminals.

## v2.1
- **[ FEATURE ]** Provide native Windows binary

## v2.0
- **[ BREAKING ]** Make `--after` and `--before` filters exclusive
- **[ FEATURE ]** Add commands for manipulating files:
  - `create` for creating a new record
  - `track` for adding an entry to a record
  - `start` to track an open-ended time range
  - `stop` to close an open-ended time range
- **[ FEATURE ]** Add `--since` and `--until` filters (inclusive)
- **[ FEATURE ]** Add `--period` filter (e.g. `--period=2015` for 
  all in 2015, or `--period=2015-04` for all in April 2015).

## v1.6
- **[ FEATURE ]** Add `json` subcommand that allows users to build
  programmatic extensions
- **[ FEATURE ]** Support Windows line endings (`\r\n`)
- **[ FEATURE ]** Add `bookmark unset` command for clearing current selection
- **[ FEATURE ]** Check stdin for input (to allow shell piping)

## v1.5
- **[ FIX ]** Fix the ongoing time counter in `klog now --follow`

## v1.4
- **[ FIX ]** Fix the ongoing time counter in the MacOS widget

## v1.3
- **[ BREAKING ]** Change structure of the bookmark subcommand
  (This is in order to account for the increasing number of operations)
- **[ FEATURE ]** Add subcommand `now` for displaying an ongoing total
  that takes open ranges into account (based on the time of execution)
- **[ FEATURE ]** Add `--now` flag to `total` and `report` to take
  open ranges into account optionally
- **[ FEATURE ]** Add subcommand `bookmark edit` for opening a bookmarked
  file in your $EDITOR
- **[ FEATURE ]** Allow to sort results in both directions
  (`--sort ASC` or `--sort DESC`)
- **[ FEATURE ]** Print warning when unclosed open ranges are detected
  in records before yesterday. (It’s probably always a mistake, if that occurs.)
  You can disable this check with the `--no-warn` flag.
- **[ FEATURE ]** Support filtering for `tags` and `reports`
- **[ FEATURE ]** Define shorthand flags, e.g. for `--now`, `--diff`
- **[ FIX ]** Don’t demand `.klg` file extensions for bookmarks

## v1.2
- **[ INFO ]** Provided more helpful error messages
- **[ FIX ]** Fix unhandled error with experimental `template` subcommand
  (introduced in v1.1)

## v1.1
- **[ INFO ]** Introduced hidden and experimental `template` subcommand,
  see https://github.com/jotaen/klog/pull/12
- **[ FIX ]** If a duration consists hours and minutes,
  the minutes cannot be greater than `59m`, e.g. `1h59m`
- **[ FIX ]** Ensure there is a final blank line when `print`-ing
- **[ FIX ]** Improve error messages regarding the bookmark subcommand

## v1.0
- **[ BREAKING ]** Renamed subcommand `eval` to `total`.
  (This wording is more inline with the documentation and
  therefore more intuitive.)
- **[ FEATURE ]** Added subcommand `report` that generates a
  calendar overview
- **[ FEATURE ]** Added subcommand `tags` that shows the total
  times aggregated by tags
- **[ FEATURE ]** Added subcommand `bookmark` (a file that
  is used by default when no input files are specified)
